@@ -1,5 +1,5 @@
 /**
- * Copyright 2005-2016 The Kuali Foundation
+ * Copyright 2005-2018 The Kuali Foundation
  *
  * Licensed under the Educational Community License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,30 +15,6 @@
  */
 package org.kuali.rice.krad.service.impl;
 
-import org.apache.commons.lang.StringUtils;
-import org.apache.ojb.broker.OptimisticLockException;
-import org.kuali.rice.core.api.util.RiceConstants;
-import org.kuali.rice.kim.api.KimConstants.PermissionNames;
-import org.kuali.rice.kim.api.identity.Person;
-import org.kuali.rice.kim.api.identity.PersonService;
-import org.kuali.rice.kim.api.permission.PermissionService;
-import org.kuali.rice.kim.api.services.KimApiServiceLocator;
-import org.kuali.rice.kns.authorization.AuthorizationConstants;
-import org.kuali.rice.krad.UserSession;
-import org.kuali.rice.krad.document.Document;
-import org.kuali.rice.krad.document.authorization.PessimisticLock;
-import org.kuali.rice.krad.exception.AuthorizationException;
-import org.kuali.rice.krad.exception.PessimisticLockingException;
-import org.kuali.rice.krad.service.BusinessObjectService;
-import org.kuali.rice.krad.service.DataDictionaryService;
-import org.kuali.rice.krad.service.KRADServiceLocatorWeb;
-import org.kuali.rice.krad.service.PessimisticLockService;
-import org.kuali.rice.krad.util.GlobalVariables;
-import org.kuali.rice.krad.util.KRADConstants;
-import org.kuali.rice.krad.util.KRADPropertyConstants;
-import org.kuali.rice.krad.util.ObjectUtils;
-import org.springframework.transaction.annotation.Transactional;
-
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -48,6 +24,29 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.apache.commons.lang.StringUtils;
+import org.kuali.rice.core.api.criteria.QueryByCriteria;
+import org.kuali.rice.core.api.util.RiceConstants;
+import org.kuali.rice.kim.api.KimConstants.PermissionNames;
+import org.kuali.rice.kim.api.identity.Person;
+import org.kuali.rice.kim.api.identity.PersonService;
+import org.kuali.rice.kim.api.permission.PermissionService;
+import org.kuali.rice.kim.api.services.KimApiServiceLocator;
+import org.kuali.rice.kns.authorization.AuthorizationConstants;
+import org.kuali.rice.krad.data.DataObjectService;
+import org.kuali.rice.krad.document.Document;
+import org.kuali.rice.krad.document.authorization.PessimisticLock;
+import org.kuali.rice.krad.exception.AuthorizationException;
+import org.kuali.rice.krad.exception.PessimisticLockingException;
+import org.kuali.rice.krad.service.DataDictionaryService;
+import org.kuali.rice.krad.service.LegacyDataAdapter;
+import org.kuali.rice.krad.service.PessimisticLockService;
+import org.kuali.rice.krad.util.GlobalVariables;
+import org.kuali.rice.krad.util.KRADConstants;
+import org.kuali.rice.krad.util.KRADPropertyConstants;
+import org.springframework.beans.factory.annotation.Required;
+import org.springframework.transaction.annotation.Transactional;
+
 /**
  * Service implementation for pessimistic locking
  *
@@ -57,22 +56,21 @@ import java.util.Set;
 public class PessimisticLockServiceImpl implements PessimisticLockService {
     private static final org.apache.log4j.Logger LOG = org.apache.log4j.Logger.getLogger(PessimisticLockServiceImpl.class);
 
-    private PersonService personService;
-    private BusinessObjectService businessObjectService;
-    private DataDictionaryService dataDictionaryService;
-    private PermissionService permissionService;
+    protected DataObjectService dataObjectService;
+    protected DataDictionaryService dataDictionaryService;
+    protected PermissionService permissionService;
+    protected PersonService personService;
 
-    /**
+	/**
      * @see org.kuali.rice.krad.service.PessimisticLockService#delete(java.lang.String)
      */
-    public void delete(String id) {
+    @Override
+	public void delete(String id) {
         if (StringUtils.isBlank(id)) {
             throw new IllegalArgumentException("An invalid blank id was passed to delete a Pessimistic Lock.");
         }
-        Map<String,Object> primaryKeys = new HashMap<String,Object>();
-        primaryKeys.put(KRADPropertyConstants.ID, Long.valueOf(id));
-        PessimisticLock lock = (PessimisticLock) getBusinessObjectService().findByPrimaryKey(PessimisticLock.class, primaryKeys);
-        if (ObjectUtils.isNull(lock)) {
+        PessimisticLock lock = dataObjectService.find(PessimisticLock.class, Long.valueOf(id));
+        if ( lock == null ) {
             throw new IllegalArgumentException("Pessimistic Lock with id " + id + " cannot be found in the database.");
         }
         Person user = GlobalVariables.getUserSession().getPerson();
@@ -86,34 +84,38 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
     	if ( LOG.isDebugEnabled() ) {
     		LOG.debug("Deleting lock: " + lock);
     	}
-        getBusinessObjectService().delete(lock);
+        dataObjectService.delete(lock);
     }
 
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#generateNewLock(String)
      */
-    public PessimisticLock generateNewLock(String documentNumber) {
+    @Override
+	public PessimisticLock generateNewLock(String documentNumber) {
         return generateNewLock(documentNumber, GlobalVariables.getUserSession().getPerson());
     }
 
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#generateNewLock(java.lang.String)
      */
-    public PessimisticLock generateNewLock(String documentNumber, String lockDescriptor) {
+    @Override
+	public PessimisticLock generateNewLock(String documentNumber, String lockDescriptor) {
         return generateNewLock(documentNumber, lockDescriptor, GlobalVariables.getUserSession().getPerson());
     }
 
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#generateNewLock(java.lang.String, org.kuali.rice.kim.api.identity.Person)
      */
-    public PessimisticLock generateNewLock(String documentNumber, Person user) {
+    @Override
+	public PessimisticLock generateNewLock(String documentNumber, Person user) {
         return generateNewLock(documentNumber, PessimisticLock.DEFAULT_LOCK_DESCRIPTOR, user);
     }
 
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#generateNewLock(java.lang.String, java.lang.String, org.kuali.rice.kim.api.identity.Person)
      */
-    public PessimisticLock generateNewLock(String documentNumber, String lockDescriptor, Person user) {
+    @Override
+	public PessimisticLock generateNewLock(String documentNumber, String lockDescriptor, Person user) {
         PessimisticLock lock = new PessimisticLock(documentNumber, lockDescriptor, user, GlobalVariables.getUserSession());
         lock = save(lock);
         if ( LOG.isDebugEnabled() ) {
@@ -125,25 +127,28 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#getPessimisticLocksForDocument(java.lang.String)
      */
-    public List<PessimisticLock> getPessimisticLocksForDocument(String documentNumber) {
-        Map fieldValues = new HashMap();
-        fieldValues.put(KRADPropertyConstants.DOCUMENT_NUMBER, documentNumber);
-        return (List<PessimisticLock>) getBusinessObjectService().findMatching(PessimisticLock.class, fieldValues);
+    @Override
+	public List<PessimisticLock> getPessimisticLocksForDocument(String documentNumber) {
+        return new ArrayList<PessimisticLock>( dataObjectService.findMatching(PessimisticLock.class,
+        		QueryByCriteria.Builder.forAttribute(KRADPropertyConstants.DOCUMENT_NUMBER, documentNumber)
+        		.build()).getResults() );
     }
 
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#getPessimisticLocksForSession(java.lang.String)
      */
-    public List<PessimisticLock> getPessimisticLocksForSession(String sessionId) {
-        Map fieldValues = new HashMap();
-        fieldValues.put(KRADPropertyConstants.SESSION_ID, sessionId);
-        return (List<PessimisticLock>) getBusinessObjectService().findMatching(PessimisticLock.class, fieldValues);
+    @Override
+	public List<PessimisticLock> getPessimisticLocksForSession(String sessionId) {
+        return new ArrayList<PessimisticLock>( dataObjectService.findMatching(PessimisticLock.class,
+        		QueryByCriteria.Builder.forAttribute(KRADPropertyConstants.SESSION_ID, sessionId)
+        		.build()).getResults() );
     }
 
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#isPessimisticLockAdminUser(org.kuali.rice.kim.api.identity.Person)
      */
-    public boolean isPessimisticLockAdminUser(Person user) {
+    @Override
+	public boolean isPessimisticLockAdminUser(Person user) {
     	return getPermissionService().isAuthorized( user.getPrincipalId(), KRADConstants.KNS_NAMESPACE, PermissionNames.ADMIN_PESSIMISTIC_LOCKING,
                 Collections.<String, String>emptyMap() );
     }
@@ -151,14 +156,15 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#releaseAllLocksForUser(java.util.List, org.kuali.rice.kim.api.identity.Person)
      */
-    public void releaseAllLocksForUser(List<PessimisticLock> locks, Person user) {
+    @Override
+	public void releaseAllLocksForUser(List<PessimisticLock> locks, Person user) {
         for (Iterator<PessimisticLock> iterator = locks.iterator(); iterator.hasNext();) {
-            PessimisticLock lock = (PessimisticLock) iterator.next();
+            PessimisticLock lock = iterator.next();
             if (lock.isOwnedByUser(user)) {
                 try {
                     delete(lock);
                 } catch ( RuntimeException ex ) {
-                    if ( ex.getCause() instanceof OptimisticLockException) {
+                	if ( ex.getCause() != null && ex.getCause().getClass().equals( LegacyDataAdapter.OPTIMISTIC_LOCK_OJB_EXCEPTION_CLASS ) ) {
                         LOG.warn( "Suppressing Optimistic Lock Exception. Document Num: " +  lock.getDocumentNumber());
                     } else {
                         throw ex;
@@ -171,14 +177,15 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#releaseAllLocksForUser(java.util.List, org.kuali.rice.kim.api.identity.Person, java.lang.String)
      */
-    public void releaseAllLocksForUser(List<PessimisticLock> locks, Person user, String lockDescriptor) {
+    @Override
+	public void releaseAllLocksForUser(List<PessimisticLock> locks, Person user, String lockDescriptor) {
         for (Iterator<PessimisticLock> iterator = locks.iterator(); iterator.hasNext();) {
-            PessimisticLock lock = (PessimisticLock) iterator.next();
+            PessimisticLock lock = iterator.next();
             if ( (lock.isOwnedByUser(user)) && (lockDescriptor.equals(lock.getLockDescriptor())) ) {
                 try {
                     delete(lock);
                 } catch ( RuntimeException ex ) {
-                    if ( ex.getCause() instanceof OptimisticLockException ) {
+                	if ( ex.getCause() != null && ex.getCause().getClass().equals( LegacyDataAdapter.OPTIMISTIC_LOCK_OJB_EXCEPTION_CLASS ) ) {
                         LOG.warn( "Suppressing Optimistic Lock Exception. Document Num: " +  lock.getDocumentNumber());
                     } else {
                         throw ex;
@@ -191,27 +198,152 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
     /**
      * @see org.kuali.rice.krad.service.PessimisticLockService#save(org.kuali.rice.krad.document.authorization.PessimisticLock)
      */
-    public PessimisticLock save(PessimisticLock lock) {
+    @Override
+	public PessimisticLock save(PessimisticLock lock) {
     	if ( LOG.isDebugEnabled() ) {
     		LOG.debug("Saving lock: " + lock);
     	}
-        return (PessimisticLock)getBusinessObjectService().save(lock);
-    }
-
-    public BusinessObjectService getBusinessObjectService() {
-        return this.businessObjectService;
-    }
-
-    public void setBusinessObjectService(BusinessObjectService businessObjectService) {
-        this.businessObjectService = businessObjectService;
+        return dataObjectService.save(lock);
     }
 
     /**
-     * @param document
-     * @param user
-     * @return Set of actions are permitted the given user on the given document
+     * {@inheritDoc}
      */
-    public Set getDocumentActions(Document document, Person user, Set<String> documentActions){
+    @Override
+    public boolean establishPessimisticLocks(Document document, Person user, boolean canEdit) {
+        // establish pessimistic locks if needed
+        if (isPessimisticLockNeeded(document, user, canEdit)) {
+            PessimisticLock pessimisticLock = createNewPessimisticLock(document, user);
+            document.addPessimisticLock(pessimisticLock);
+        }
+
+        // find if any pessimistic locks are owned by the given user
+        for (PessimisticLock pessimisticLock : document.getPessimisticLocks()) {
+            if (pessimisticLock.isOwnedByUser(user)) {
+                return true;
+            }
+        }
+
+        return false;
+    }
+
+    /**
+     * Determines whether to add a pessimistic lock on the {@code document} for {@code user} based on the current
+     * pessimistic locks and edit modes.
+     *
+     * @param document the document on which the locks will be established
+     * @param user the user for which the locks are being established
+     * @param canEdit whether the user currently can edit the document
+     *
+     * @return true if a pessimistic lock should be added, false otherwise
+     */
+    protected boolean isPessimisticLockNeeded(Document document, Person user, boolean canEdit) {
+        List<String> userOwnedLockDescriptors = new ArrayList<String>();
+        Map<String, Set<String>> otherOwnedLockDescriptors = new HashMap<String,Set<String>>();
+
+        // create the two lock containers that help determine whether to add a pessimistic lock
+        for (PessimisticLock pessimisticLock : document.getPessimisticLocks()) {
+            if (pessimisticLock.isOwnedByUser(user)) {
+                userOwnedLockDescriptors.add(pessimisticLock.getLockDescriptor());
+            } else {
+                if (!otherOwnedLockDescriptors.containsKey(pessimisticLock.getLockDescriptor())) {
+                    otherOwnedLockDescriptors.put(pessimisticLock.getLockDescriptor(), new HashSet<String>());
+                }
+
+                String otherOwnerPrincipalId = pessimisticLock.getOwnedByUser().getPrincipalId();
+                otherOwnedLockDescriptors.get(pessimisticLock.getLockDescriptor()).add(otherOwnerPrincipalId);
+            }
+        }
+
+        // double check whether the existing pessimistic locks are sane
+        checkExistingPessimisticLocks(document, userOwnedLockDescriptors, otherOwnedLockDescriptors);
+
+        // if no one has a lock on this document, then the document can be locked if the user can edit it
+        if (userOwnedLockDescriptors.isEmpty() && otherOwnedLockDescriptors.isEmpty()) {
+            return canEdit;
+        }
+
+        // if custom locking is turned on, then if the current user doesn't have a custom lock on this document and no
+        // one else has a custom lock on this document, then the document can be locked if the user can edit it
+        if (document.useCustomLockDescriptors()) {
+            String customLockDescriptor = document.getCustomLockDescriptor(user);
+            boolean userOwnsCustomLockDescriptor = userOwnedLockDescriptors.contains(customLockDescriptor);
+            boolean otherOwnsCustomLockDescriptor = otherOwnedLockDescriptors.containsKey(customLockDescriptor);
+
+            if (!userOwnsCustomLockDescriptor && !otherOwnsCustomLockDescriptor) {
+                return canEdit;
+            }
+        }
+
+        return false;
+    }
+
+    /**
+     * Check to make sure that the current user doesn't erroneously share the same lock with another user.
+     *
+     * @param document the document to check for erroneous locks
+     * @param userOwnedLockDescriptors the list of lock descriptors for the current user
+     * @param otherOwnedLockDescriptors the map of other lock descriptors to all of the other users that own them
+     */
+    protected void checkExistingPessimisticLocks(Document document, List<String> userOwnedLockDescriptors, Map<String, Set<String>> otherOwnedLockDescriptors) {
+        for (String userOwnedLockDescriptor : userOwnedLockDescriptors) {
+            if (otherOwnedLockDescriptors.containsKey(userOwnedLockDescriptor)) {
+                Set<String> otherOwnerPrincipalIds = otherOwnedLockDescriptors.get(userOwnedLockDescriptor);
+                String workflowOwnerPrincipalId = getWorkflowPessimisticLockOwnerUser().getPrincipalId();
+                boolean hasOtherOwners = !otherOwnerPrincipalIds.isEmpty();
+                boolean hasMoreThanOneOtherOwner = otherOwnerPrincipalIds.size() > 1;
+                boolean hasWorkflowOwner = otherOwnerPrincipalIds.contains(workflowOwnerPrincipalId);
+
+                // if the lock has other owners, then if there is more than one other owner or if the single other owner
+                // is not the workflow user, then throw an error
+                if (hasOtherOwners && (hasMoreThanOneOtherOwner || !hasWorkflowOwner)) {
+                    StringBuilder builder = new StringBuilder("Found an invalid lock status on document number ");
+                    builder.append(document.getDocumentNumber());
+                    builder.append(" with current user and other user both having locks concurrently");
+
+                    if (document.useCustomLockDescriptors()) {
+                        builder.append(" for custom lock descriptor '");
+                        builder.append(userOwnedLockDescriptor);
+                        builder.append("'");
+                    }
+
+                    builder.append(".");
+
+                    LOG.debug(builder.toString());
+
+                    throw new PessimisticLockingException(builder.toString());
+                }
+            }
+        }
+    }
+
+    /**
+     * Creates a new {@link PessimisticLock} on a {@code document} for a {@code user}.
+     *
+     * <p>
+     * If the {@code document} uses custom lock descriptors, then the new lock is generated with a custom lock
+     * descriptor for a portion of the document.  Otherwise, it will create a lock over the entire document.
+     * </p>
+     *
+     * @param document the document on which the locks will be established
+     * @param user the user for which the locks are being established
+     *
+     * @return the newly created lock object
+     */
+    protected PessimisticLock createNewPessimisticLock(Document document, Person user) {
+        if (document.useCustomLockDescriptors()) {
+            return generateNewLock(document.getDocumentNumber(), document.getCustomLockDescriptor(user), user);
+        } else {
+            return generateNewLock(document.getDocumentNumber(), user);
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    @Deprecated
+	public Set getDocumentActions(Document document, Person user, Set<String> documentActions){
     	if(documentActions.contains(KRADConstants.KUALI_ACTION_CAN_CANCEL) && !hasPreRouteEditAuthorization(document, user) ){
     		documentActions.remove(KRADConstants.KUALI_ACTION_CAN_CANCEL);
     	}
@@ -237,13 +369,16 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
      *         {@link #hasInitiateAuthorization(Document, Person)}), and the document has a lock owned by the given
      *         user. If the document is not using Pessimistic Locking the value returned will be that returned by
      *         {@link #hasInitiateAuthorization(Document, Person)}.
+     *
+     * @deprecated Different mechanism of obtaining authorization in KRAD
      */
+    @Deprecated
     protected boolean hasPreRouteEditAuthorization(Document document, Person user) {
     	if (document.getPessimisticLocks().isEmpty()) {
     		return true;
     	}
-    	for (Iterator iterator = document.getPessimisticLocks().iterator(); iterator.hasNext();) {
-    		PessimisticLock lock = (PessimisticLock) iterator.next();
+    	for (Iterator<PessimisticLock> iterator = document.getPessimisticLocks().iterator(); iterator.hasNext();) {
+    		PessimisticLock lock = iterator.next();
     		if (lock.isOwnedByUser(user)) {
     			return true;
             }
@@ -251,7 +386,7 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
         return false;
     }
 
-
+    @Deprecated
     protected boolean usesPessimisticLocking(Document document) {
         return getDataDictionaryService().getDataDictionary().getDocumentEntry(document.getClass().getName()).getUsePessimisticLocking();
     }
@@ -263,8 +398,9 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
      * @param document - the document to create the lock against and add the lock to
      * @see org.kuali.rice.kns.document.authorization.DocumentAuthorizer#establishWorkflowPessimisticLocking(org.kuali.rice.krad.document.Document)
      */
-    public void establishWorkflowPessimisticLocking(Document document) {
-        PessimisticLock lock = createNewPessimisticLock(document, new HashMap(), getWorkflowPessimisticLockOwnerUser());
+    @Override
+	public void establishWorkflowPessimisticLocking(Document document) {
+        PessimisticLock lock = createNewPessimisticLock(document, getWorkflowPessimisticLockOwnerUser());
         document.addPessimisticLock(lock);
     }
 
@@ -274,7 +410,8 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
      * @param document - document to release locks from
      * @see org.kuali.rice.kns.document.authorization.DocumentAuthorizer#releaseWorkflowPessimisticLocking(org.kuali.rice.krad.document.Document)
      */
-    public void releaseWorkflowPessimisticLocking(Document document) {
+    @Override
+	public void releaseWorkflowPessimisticLocking(Document document) {
         releaseAllLocksForUser(document.getPessimisticLocks(), getWorkflowPessimisticLockOwnerUser());
         document.refreshPessimisticLocks();
     }
@@ -294,6 +431,8 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
     }
 
     /**
+     * {@inheritDoc}
+     *
      * This implementation will check the given document, editMode map, and user object to verify Pessimistic Locking. If the
      * given edit mode map contains an 'entry type' edit mode then the system will check the locks already in existence on
      * the document. If a valid lock for the given user is found the system will return the given edit mode map. If a valid
@@ -306,7 +445,9 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
      * @see org.kuali.rice.kns.document.authorization.DocumentAuthorizer#establishLocks(org.kuali.rice.krad.document.Document,
      *      java.util.Map, org.kuali.rice.kim.api.identity.Person)
      */
-    public Map establishLocks(Document document, Map editMode, Person user) {
+    @Override
+    @Deprecated
+	public Map establishLocks(Document document, Map editMode, Person user) {
         Map editModeMap = new HashMap();
         // givenUserLockDescriptors is a list of lock descriptors currently held on the document by the given user
         List<String> givenUserLockDescriptors = new ArrayList<String>();
@@ -323,7 +464,7 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
                 if (!lockDescriptorUsers.containsKey(lock.getLockDescriptor())) {
                     lockDescriptorUsers.put(lock.getLockDescriptor(), new HashSet<Person>());
                 }
-                ((Set<Person>) lockDescriptorUsers.get(lock.getLockDescriptor())).add(lock.getOwnedByUser());
+                lockDescriptorUsers.get(lock.getLockDescriptor()).add(lock.getOwnedByUser());
             }
         }
 
@@ -410,6 +551,7 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
      *            user the lock will be 'owned' by
      * @return true if the given edit mode map has at least one 'entry type' edit mode... false otherwise
      */
+    @Deprecated
     protected boolean isLockRequiredByUser(Document document, Map editMode, Person user) {
         // check for entry edit mode
         for (Iterator iterator = editMode.entrySet().iterator(); iterator.hasNext();) {
@@ -432,6 +574,7 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
      * @return an adjusted edit mode map where 'entry type' edit modes have been removed or replaced using the
      *         {@link #getEntryEditModeReplacementMode} method
      */
+    @Deprecated
     protected Map getEditModeWithEditableModesRemoved(Map currentEditMode) {
         Map editModeMap = new HashMap();
         for (Iterator iterator = currentEditMode.entrySet().iterator(); iterator.hasNext();) {
@@ -455,10 +598,11 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
      * @return true if the given entry has a key signifying an 'entry type' edit mode and the value is equal to
      *         {@link #EDIT_MODE_DEFAULT_TRUE_VALUE}... false if not
      */
+    @Deprecated
     protected boolean isEntryEditMode(Map.Entry entry) {
     	// check for FULL_ENTRY edit mode set to default true value
     	if (AuthorizationConstants.EditMode.FULL_ENTRY.equals(entry.getKey())) {
-    		String fullEntryEditModeValue = (String)entry.getValue();           
+    		String fullEntryEditModeValue = (String)entry.getValue();
     		return ( StringUtils.equalsIgnoreCase(KRADConstants.KUALI_DEFAULT_TRUE_VALUE, fullEntryEditModeValue) );
     	}
     	return false;
@@ -489,6 +633,7 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
      *            user who will 'own' the new lock object
      * @return the newly created lock object
      */
+    @Deprecated
     protected PessimisticLock createNewPessimisticLock(Document document, Map editMode, Person user) {
         if (document.useCustomLockDescriptors()) {
             return generateNewLock(document.getDocumentNumber(), document.getCustomLockDescriptor(user), user);
@@ -505,12 +650,14 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
     }
 
 	public DataDictionaryService getDataDictionaryService() {
-        if ( dataDictionaryService == null ) {
-        	dataDictionaryService = KRADServiceLocatorWeb.getDataDictionaryService();
-        }
 		return dataDictionaryService;
 	}
 
+	@Required
+	public void setDataDictionaryService(DataDictionaryService dataDictionaryService) {
+		this.dataDictionaryService = dataDictionaryService;
+	}
+
 	public PermissionService getPermissionService() {
         if ( permissionService == null ) {
         	permissionService = KimApiServiceLocator.getPermissionService();
@@ -519,6 +666,9 @@ public class PessimisticLockServiceImpl implements PessimisticLockService {
 	}
 
 
-
+	@Required
+    public void setDataObjectService(DataObjectService dataObjectService) {
+		this.dataObjectService = dataObjectService;
+	}
 }
 
