@@ -1,5 +1,5 @@
 /**
- * Copyright 2005-2016 The Kuali Foundation
+ * Copyright 2005-2018 The Kuali Foundation
  *
  * Licensed under the Educational Community License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@ import org.kuali.rice.krad.datadictionary.parse.BeanTag;
  *
  * @author Kuali Rice Team (rice.collab@kuali.org)
  */
-@BeanTag(name = "datePicker-bean", parent = "Uif-DatePicker")
+@BeanTag(name = "datePicker", parent = "Uif-DatePicker")
 public class DatePicker extends WidgetBase {
 	private static final long serialVersionUID = -659830874214415990L;
 
