@@ -1,5 +1,5 @@
 /**
- * Copyright 2005-2016 The Kuali Foundation
+ * Copyright 2005-2018 The Kuali Foundation
  *
  * Licensed under the Educational Community License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -392,7 +392,7 @@ public class RuleBo implements RuleDefinitionContract, Versioned, Serializable {
         for (Map.Entry<String, String> entry : im.getAttributes().entrySet()) {
             KrmsAttributeDefinitionBo attrDefBo = KrmsRepositoryServiceLocator.getKrmsAttributeDefinitionService().getKrmsAttributeBo(entry.getKey(), im.getNamespace());
             attributeBo = new RuleAttributeBo();
-            attributeBo.setRule(RuleBo.from(im));
+            attributeBo.setRule(bo);
             attributeBo.setAttributeDefinition(attrDefBo);
             attributeBo.setValue(entry.getValue());
             attributeBo.setAttributeDefinition(attrDefBo);
